#!/bin/env python
import time
import parmepy as pp
from parmepy.gpu import PARMES_REAL_GPU, PARMES_DOUBLE_GPU
from parmepy.fields.continuous import Field
from parmepy.fields.analytical import AnalyticalField
from parmepy.operator.advection import Advection
from parmepy.problem.transport import TransportProblem
from parmepy.operator.monitors.printer import Printer
import math
import sys


def run(nb=257):
    dim = 3
    boxLength = [1., 1., 1.]
    boxMin = [0., 0., 0.]
    nbElem = [nb, nb, nb]

    timeStep = 0.05
    #period = 10.
    finalTime = 40 * timeStep
    outputFilePrefix = './res_3D/levelSet_3D_'
    outputModulo = 0

    t0 = time.time()

    ## Domain
    box = pp.Box(dim, length=boxLength, origin=boxMin)

    ## Fields
    scal = Field(domain=box, name='Scalar')
    velo = Field(domain=box, name='Velocity', vector=True)
    #scal = pp.AnalyticalField(domain=box, formula=scalaire, name='Scalar')
    #velo = pp.AnalyticalField(domain=box, formula=vitesse, name='Velocity', vector=True)

    ## Operators
    advec = Advection(velo, scal,
                      resolutions={velo: nbElem,
                                   scal: nbElem},
                      #method='gpu_2k_m6prime',
                      #method='gpu_2k_m8prime',
                      #method='gpu_1k_m6prime',
                      method='gpu_1k_m8prime',
                      src=['./levelSet3D.cl'],
                      #precision=PARMES_REAL_GPU,
                      precision=PARMES_DOUBLE_GPU,
                      )
    #velocity = pp.Velocity(velo)

    ##Problem
    #pb = pp.Problem(topo3D, [velocity, advec])
    pb = TransportProblem([advec],
                          monitors=[Printer(fields=[scal, velo],
                                            frequency=outputModulo,
                                            outputPrefix=outputFilePrefix)])

    ## Setting solver to Problem
    pb.setUp(finalTime, timeStep)

    t1 = time.time()

    ## Solve problem
    timings = pb.solve()

    tf = time.time()

    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"
    print ""
    return pb.timings_info[0] + "\"Solving time\" \"Init time\" \"Total time\"", pb.timings_info[1] + str(tf - t1) + " " + str(t1 - t0) + " " + str(tf - t0)

if __name__ == "__main__":
    run(257)
    # timings = {}
    # f = open('bench_levelSet3D.dat', 'w')
    # header = "#size dim nPart "
    # sizes = xrange(32, 256+32*3+1, 32)
    # #sizes = [256+32+32+32]
    # for s in sizes:
    #     h, t = run(s)
    #     timings[s] = t
    # header += h
    # f.write(header + "\n")
    # for s in sizes:
    #     f.write(str(s) + " " + str(3) + " " + str(s**3) + " " + str(timings[s]) + "\n")
    # f.close()
