"""
2D Obstacles representation.
"""
from parmepy.constants import np, PARMES_REAL
from parmepy.domain.obstacle.obstacle import Obstacle


class Obstacle2D(Obstacle):
    """
    2D Obstacles representation.
    """

    def __init__(self, domain, 
                 center=[0.5, 0.5],
                 zlayer=0.02,
                 porousLayerThickn=0.05,
                 porousLayerConfig='',
                 obstacleName='',
                 **otherConfig):
        """
        Constructor.
        Create a 2D obstacle and z-boundaries from given parameters :

        @param domain : domain in which the obstacle is included
        @param zlayer : width of z-boundaries layer
        @param center : position of the obstacle center
        @param porousLayerThickn : thickness of the added porous layer
        @param porousLayerConfig : geometrical configuration
        of the added porous layer
        """
        Obstacle.__init__(self, domain)
        if not (self.dimension == 2):
            raise ValueError("Error, the domain is not bi-dimensional")
        ## Position of the obstacle center
        self.center = np.asarray(center, dtype=PARMES_REAL)
        ## Width of z-boundaries layer
        self.zlayer = zlayer
        ## Thickness of the porous layer at the obstacle surface
        self.porousLayerThickn = porousLayerThickn
        ## geometrical configuration of the added porous layer
        self.porousLayerConfig = porousLayerConfig
        ## Obstacle name
        self.obstacleName = obstacleName
        ## Other cbstacle configurations
        self.config = otherConfig

    def setUp(self):
        if self.definedObst is None:
            if self.obstacleName.find('semi_cylinder') >= 0:
                from parmepy.domain.obstacle.semi_cylinder import SemiCylinder
                self.definedObst = SemiCylinder(self, **self.config)
            elif self.obstacleName.find('cylinder') >= 0:
                from parmepy.domain.obstacle.cylinder import Cylinder
                self.definedObst = Cylinder(self, **self.config)
            else:
                print "Unknown obstacle '", self.obstacleName, \
                      "' --> Using default 2D obstacle : cylinder"
                from parmepy.domain.obstacle.Cylinder import Cylinder
                self.definedObst = Cylinder(self, **self.config)

if __name__ == "__main__" :
    print __doc__
    print "- Provided class : Obstacle2D"
    print Obstacle2D.__doc__
