"""
3D Obstacles representation.
"""
from parmepy.constants import np, PARMES_REAL
from parmepy.domain.obstacle.obstacle import Obstacle


class Obstacle3D(Obstacle):
    """
    3D Obstacles representation.
    """

    def __init__(self, domain, 
                 center=[0.5, 0.5, 0.5],
                 zlayer=0.02,
                 porousLayerThickn=0.,
                 porousLayerConfig='',
                 obstacleName='',
                 **otherConfig):
        """
        Constructor.
        Create a 3D obstacle and z-boundaries from given parameters :

        @param domain : domain in which the obstacle is included
        @param zlayer : width of z-boundaries layer
        @param center : position of the obstacle center
        @param porousLayerThickn : thickness of the added porous layer
        @param porousLayerConfig : geometrical configuration
        of the added porous layer
        """
        Obstacle.__init__(self, domain)
        if not (self.dimension == 3):
            raise ValueError("Error, the domain is not tri-dimensional")
        ## Position of the obstacle center
        self.center = np.asarray(center, dtype=PARMES_REAL)
        ## Width of z-boundaries layer
        self.zlayer = zlayer
        ## Thickness of the porous layer at the obstacle surface
        self.porousLayerThickn = porousLayerThickn
        ## geometrical configuration of the added porous layer
        self.porousLayerConfig = porousLayerConfig
        ## Obstacle name
        self.obstacleName = obstacleName
        ## Other cbstacle configurations
        self.config = otherConfig

    def setUp(self):
        if self.definedObst is None:
            if self.obstacleName.find('hemisphere') >= 0:
                from parmepy.domain.obstacle.hemisphere import Hemisphere
                self.definedObst = Hemisphere(self, **self.config)
            elif self.obstacleName.find('sphere') >= 0:
                from parmepy.domain.obstacle.sphere import Sphere
                self.definedObst = Sphere(self, **self.config)
            else:
                print "Unknown obstacle '", self.obstacleName, \
                      "' --> Using default 3D obstacle : sphere"
                from parmepy.domain.obstacle.sphere import Sphere
                self.obstacleName = 'sphere'
                self.definedObst = Sphere(self, **self.config)

if __name__ == "__main__" :
    print __doc__
    print "- Provided class : Obstacle3D"
    print Obstacle3D.__doc__
