"""
@package parmepy.fields.analytical

Continuous variable description defined with an analytic formula.
"""
from parmepy.constants import debug
from parmepy.fields.continuous import Field


class AnalyticalField(Field):
    """
    A variable (continuous) defined with an analytic formula.

    """

    @debug
    def __init__(self, domain, formula, name="", vector=False):
        """
        Create an AnalyticalField from a formula.

        @param domain : variable application domain.
        @param formula : python function
        @param name : name of the variable (used for vtk output).
        @param vector : is variable is a vector.

        @note formula is used in ScalarField or VectorField as vectorized function by numpy.
        """
        Field.__init__(self, domain, name, vector)
        ## Analytic formula.
        self.formula = formula

    def value(self, *pos):
        """
        Evaluation of the variable at a given position.

        @param pos : Position to evaluate.
        @return formula(pos): value of the formula at the given position.
        """
        return self.formula(*pos)

    @debug
    def initialize(self):
        if self._fieldId == -1:
            raise ValueError("Cannot initialise analytical field non discretized.")
        print self.name,
        for dF in self.discreteField:
            dF.initialize(self.formula)
        print "Done"

    def __str__(self):
        """ToString method"""
        s = "  Analytical variable (ContinuousField)\n"
        s += Field.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : AnalyticalField"
    print AnalyticalField.__doc__
