"""
@package parmepy.fields.discrete

Discretes variables (scalar and vectors) descriptions.
"""
from abc import ABCMeta, abstractmethod
from parmepy.constants import debug


class DiscreteField(object):
    """Abstract description of discrete field. """

    __metaclass__ = ABCMeta

    @debug
    def __new__(cls, *args, **kw):
        return object.__new__(cls, *args, **kw)

    @debug
    @abstractmethod
    def __init__(self, parent, topology=None, name="?", idFromParent=None):
        """ Creates a ScalaField.

        @param parent : Continuous field.
        @param topology : Topology informations
        @param name : Field name
        @param idFromParent : Index in the parent's discrete fields
        """
        if(topology is not None):
            ## Topology informations.
            self.topology = topology
        else:
            raise NotImplementedError()
        ## Field name.
        self.name = name
        ## Field dimension.
        self.dimension = topology.domain.dimension
        ## Field resolution.
        self.resolution = self.topology.mesh.resolution
        ## @private Continuous field
        self.__parentField = parent
        ## @private Index in parent's field discrete fields
        self.__idFromParent = idFromParent
        ## Application domain of the variable.
        self.domain = self.topology.mesh
        ## Is self.data contains data
        self.contains_data = False
        ## Scalar is not vector
        self.vector = None
        ## Field data numpy array.
        self.data = None

    @abstractmethod
    def __getitem__(self, i):
        """ Access to the content of the field."""
        raise ValueError("Abstract method")

    @abstractmethod
    def __setitem__(self, i, value):
        """ Access to the content of the field."""
        raise ValueError("Abstract method")

    @debug
    def initialize(self, formula=None):
        """Initialize values with given formula."""
        raise ValueError("Abstract method")

    def get_data_method(self):
        pass


if __name__ == "__main__":
    print __doc__
    print "- Provided class : Domain (abstract)."
    print Domain.__doc__





