"""
Testing parmepy.field.continuous.Field
"""
import parmepy as pp
from parmepy.fields.continuous import Field
from parmepy.fields.analytical import AnalyticalField


def test_discretization():
    """Test multiple discretizations"""
    dom = pp.box.Box()
    cf = Field(dom)
    resolTopo = [33, 33, 17]
    topo = pp.topology.Cartesian(dom, 3, resolTopo)
    cf.discretize(topo)
    assert cf._fieldId == 0
    assert len(cf.discreteField) == 1
    assert len(cf.topologies) == 1
    assert cf.topologies[0] == topo
    cf.discretize(topo)
    assert cf._fieldId == 0
    assert len(cf.discreteField) == 1
    assert len(cf.topologies) == 1
    assert cf.topologies[0] == topo
    resolTopo = [11, 17, 19]
    topo2 = pp.topology.Cartesian(dom, 3, resolTopo)
    cf.discretize(topo2)
    assert cf._fieldId == 1
    assert len(cf.discreteField) == 2
    assert len(cf.topologies) == 2
    assert cf.topologies[0] == topo
    assert cf.topologies[1] == topo2


def test_analytical():
    """Test formula"""
    dom = pp.box.Box()
    formula = lambda x, y, z: (x * y * z, 1.0, -x * y * z)
    caf = AnalyticalField(dom, formula)
    assert caf.value(0., 0., 0.) == (0., 1., 0.)
    assert caf.value(1., 1., 1.) == (1., 1., -1.)
    assert caf.value(1., 2., 3.) == (6., 1., -6.)
