
inline uint noBC_id(int id, int nb_part){
  return (id%nb_part)*WGNADVECANDREMESH+(id/nb_part);
}
__kernel void advection_and_remeshing(__global const float* gvelo,
				      __global const float* pscal,
				      __global float* gscal,
				      float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  float invdx = 1.0/dx;
  int4 ind, i_ind, i_ind_p;
  uint i, nb_part = WIDTH/WGNADVECANDREMESH;
  float4 p, s, y, gs, v, vp, coord;
  uint4 index4;

  __local float gscal_loc[WIDTH];
  __local float gvelo_loc[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=(WGNADVECANDREMESH*4))
    {
      v = vload4((i+gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, gvelo);
      gvelo_loc[noBC_id(i, nb_part)] = v.x;
      gvelo_loc[noBC_id(i+1, nb_part)] = v.y;
      gvelo_loc[noBC_id(i+2, nb_part)] = v.z;
      gvelo_loc[noBC_id(i+3, nb_part)] = v.w;
      gscal_loc[i] = 0.0;
      gscal_loc[i+1] = 0.0;
      gscal_loc[i+2] = 0.0;
      gscal_loc[i+3] = 0.0;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*nb_part; i<(gidX + 1)*nb_part; i+=4)
    {
      v = (float4)(gvelo_loc[noBC_id(i,nb_part)],
                   gvelo_loc[noBC_id(i+1,nb_part)],
                   gvelo_loc[noBC_id(i+2,nb_part)],
                   gvelo_loc[noBC_id(i+3,nb_part)]);
      coord = (float4)(i*dx, (i+1)*dx, (i+2)*dx, (i+3)*dx);
      p = fma((float4)(0.5*dt),v,coord);
      ind = convert_int4_rtn(p * invdx);
      y = (fma(- convert_float4(ind),dx,p))* invdx ;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((i_ind + 1) % WIDTH);
      v = (float4)(gvelo_loc[noBC_id(i_ind.x,nb_part)], gvelo_loc[noBC_id(i_ind.y,nb_part)],
                   gvelo_loc[noBC_id(i_ind.z,nb_part)], gvelo_loc[noBC_id(i_ind.w,nb_part)]);
      vp = (float4)(gvelo_loc[noBC_id(i_ind_p.x,nb_part)], gvelo_loc[noBC_id(i_ind_p.y,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.z,nb_part)], gvelo_loc[noBC_id(i_ind_p.w,nb_part)]);
      p=fma(mix(v,vp,y),dt,coord);

      s = vload4((i + gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, pscal);
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      index4 = convert_uint4((ind - 3 + WIDTH) % WIDTH);
      gs =  (alpha(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (beta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs = (gamma(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (delta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (eta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (zeta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (theta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (iota(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*4; i<WIDTH; i+=(WGNADVECANDREMESH*4))
    vstore4((float4)(gscal_loc[noBC_id(i, nb_part)],gscal_loc[noBC_id(i+1, nb_part)], gscal_loc[noBC_id(i+2, nb_part)], gscal_loc[noBC_id(i+3, nb_part)]), (i + gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, gscal);
}
