"""
@package parmepy.numerics.integrators.odesolver

Classes to describe (time) integrators.
"""
from abc import ABCMeta, abstractmethod
from parmepy.constants import np
from parmepy.numerics.method import NumMethod


class ODESolver(NumMethod):
    """
    Abstract description for ODE solvers.
    Solve the system :
    y'(t) = f(t,y)

    By default, f = 0.
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, f=None):
        """
        Constructor.

        @param f function f(t,y) : Right hand side of the equation to solve.
        """
        super(ODESolver, self).__init__()
        ## RHS.
        if f is not None:
            self.f = f
        else:
            #[0. for x in y]
            self.f = lambda t, y: np.vectorize(f)(t, y)

    @abstractmethod
    def __call__(self, y, t, dt):
        """
        Abstract method, apply operaton on a variable.
        Must be implemented by sub-class.

        @param y : position at time t.
        @param t : current time.
        @param dt : time step.
        @return y at t+dt.
        """
        pass

if __name__ == "__main__":
    print __doc__
    print "- Provided class : ODESolver (abstract)"
    print ODESolver.__doc__




