"""
@package parmepy.numerics.remeshing.remeshing
"""
from abc import ABCMeta, abstractmethod
from parmepy.constants import np, PARMES_REAL, ORDER
from parmepy.numerics.method import NumMethod


class Remeshing(NumMethod):
    """Remshing with M6prime function"""

    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, dim):
        NumMethod.__init__(self)
        self.name = 'M6Prime'
        self._slice_all = [slice(None, None, None)
                            for d in xrange(dim)]

    def slice_i_along_d(self, i, d):
        l = list(self._slice_all)
        l[d] = i
        return tuple(l)

    @abstractmethod
    def __call__(self, ppos, pscal, d):
        """
        Remesh particles at position ppos with scalar
        pscal along direction d.

        @param ppos : particle position
        @param pscal : particle scalar
        @param d : splitting direction
        """
        pass
