"""
@package parmepy.operator.analytic_d

Analytic discrete operator representation.
"""
from parmepy.constants import np, debug, ORDER
from parmepy.gpu import cl
from parmepy.operator.discrete import DiscreteOperator
import time


class Analytic_D(DiscreteOperator):
    """
    Analytic computation from analytical expression.x
    """

    @debug
    def __init__(self, analytic, method=''):
        """
        Create a Velocity operator.

        @param velocity_Op : Velocity operator.
        @param idVelocityD : Index of velocity discretisation to use.
        @param method : the method to use.
        """
        ## Velocity.
        self.method = method
        fields = [f.discreteField[analytic.discreteFieldId[f]]
                  for f in analytic.variables]
        DiscreteOperator.__init__(self, fields)
        self.input = []
        self.output = fields
        self.compute_time = 0.
        self.name = "analytic"
        self.formula = analytic.formula

    @debug
    def setUp(self, method=''):
        ## numpy formula vectorization
        self.vfomula = np.vectorize(self.formula)

    @debug
    def apply(self, t, dt, ite):
        cTime = time.time()
        for df in self.variables:
            args = list(df.topology.mesh.coords)
            args.append(t)
            args.append(dt)
            args.append(ite)
            datatype = df.data[0].dtype
            if df.topology.dim == 3:
                df.data[0], df.data[1], df.data[2] = self.vfomula(*args)
                df.data[0] = np.asarray(df.data[0],
                                        dtype=datatype, order=ORDER)
                df.data[1] = np.asarray(df.data[1],
                                        dtype=datatype, order=ORDER)
                df.data[2] = np.asarray(df.data[2],
                                        dtype=datatype, order=ORDER)
            elif df.topology.dim == 2:
                df.data[0], df.data[1] = self.vfomula(*args)
                df.data[0] = np.asarray(df.data[0],
                                        dtype=datatype, order=ORDER)
                df.data[1] = np.asarray(df.data[1],
                                        dtype=datatype, order=ORDER)
            else:
                df.data = self.vfomula(*args)
                df.data = np.asarray(df.data,
                                     dtype=datatype, order=ORDER)
        for df in self.output:
            df.contains_data = True
        cTime = time.time() - cTime
        self.compute_time += cTime
        return cTime

    def printComputeTime(self):
        print "Velocity total time : ", self.compute_time

    def __str__(self):
        s = "Velocity_P (DiscreteOperator). " + DiscreteOperator.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : AdvectionDOp"
    print AdvectionDOp.__doc__
