"""
@package parmepy.operator.continuous

Operator representation.
"""
from abc import ABCMeta, abstractmethod
from parmepy.constants import debug


class Operator(object):
    """
    Continuous operator abstract description.
    """

    __metaclass__ = ABCMeta

    @debug
    def __new__(cls, *args, **kw):
        return object.__new__(cls, *args, **kw)

    @debug
    @abstractmethod
    def __init__(self, variables):
        """
        Create a ContinuousOperator.
        """
        ## Variables
        self.variables = variables
        ## Domain
        self.domain = self.variables[0].domain
        ## Fields discretes id
        self.discreteFieldId = {}
        ## Operator discretization.
        self.discreteOperator = None
        ## Timings informations
        self.time_info = ["", ""]

    def addVariable(self, cVariable):
        """
        Add an continuous variables to the operator.

        @param cVariable : list of variables to add.
        """
        for v in cVariable:
            if self.variables.count(v) == 0:
                self.variables.append(v)

    @debug
    def setUp(self):
        if self.resolutions is None:
            raise ValueError("Advection setUp: no resolution given.")

    @debug
    def finalize(self):
        """
        Method called on simulation finalization.
        Clean operator at time loop end.
        """
        self.discreteOperator.finalize()

    @debug
    def apply(self, *args):
        """
        Apply operator.

        @param *args : Operator arguments.
        @return computing time.
        """
        return self.discreteOperator.apply(*args)

    def printComputeTime(self):
        """Displays compute time for operator."""
        if self.discreteOperator is not None:
            self.discreteOperator.printComputeTime()
            self.time_info = self.discreteOperator.time_info
        else:
            raise ValueError("Cannot print compute time" +
                             "of a non discretized operator")

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Operator"
    print ContinuousOperator.__doc__
