# -*- coding: utf-8 -*-
"""
@package parmepy.operator.diffusion
Continous Diffusion operator (F2PY)
"""
from parmepy.operator.continuous import Operator
from parmepy.f2py import fftw2py
from parmepy.mpi.topology import Cartesian
from diffusion_fft import DiffusionFFT
from parmepy.constants import debug


class Diffusion(Operator):
    """
    Diffusion operator representation using FFT
    """

    @debug
    def __init__(self, velocity=None, vorticity=None,
                 resolutions=None, method='',
                 **other_config):
        """
        Constructor.
        Create a Diffusion operator using FFT.

        @param velocity ContinuousVectorField : velocity variable.
        @param vorticity ContinuousVectorField : vorticity variable.
        @param viscosity : viscosity of the considered medium.
        """
        Operator.__init__(self, [velocity, vorticity])
        self.velocity = velocity
        self.vorticity = vorticity
        self.resolutions = resolutions
        self.method = method
        self.config = other_config

    @debug
    def setUp(self):
        """
        Diffusion operator discretization method.
        Create a discrete Diffusion operator from given specifications.
        """
        Operator.setUp(self)
        print self.method
        localres, localoffset = fftw2py.init_fftw_solver(
                                    self.resolutions[self.vorticity],
                                    self.domain.length)
        topodims = self.resolutions[self.vorticity] / localres
        print 'topodims DIFFUSION', topodims
        #variables discretization
        for v in self.variables:
            topoId, topo = self.domain.addTopology(
                Cartesian.withResolution(
                    domain=self.domain, topoResolution=topodims,
                    globalMeshResolution=self.resolutions[v]))
            df, dfId = v.discretize(topo)
            self.discreteFieldId[v] = dfId
        self.discreteOperator = DiffusionFFT(self, self.method,
                                             **self.config)
        self.discreteOperator.setUp()

    def __str__(self):
        """ToString method"""
        s = " Diffusion operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Diffusion"
    print Diffusion.__doc__
