# -*- coding: utf-8 -*-
"""
@package operator
Discrete MultiPhase Rot Grad P
"""
from parmepy.operator.discrete import DiscreteOperator
from parmepy.operator.differentialOperator import DifferentialOperator
from parmepy.operator.differentialOperator_d import DifferentialOperator_d
from parmepy.constants import np
import numpy.testing as npt
import time
import sys

class Pressure_d(DiscreteOperator):
    """
    Operator representation.
    """

    @debug
    def __init__(self,pressureOp, method=''):
        """
        Constructor.
        Create the old velocity field to compute the -Dp/rho
        in N.S equation
        @param operator.
        """
        DiscreteOperator.__init__(self)
        self.method = method

        self.velocity_old = np.copy(pressureOp.velocity.getDiscreteField(
            pressureOp.resolutions[pressureOp.velocity]))
        self.velocity_new = pressureOp.velocity.getDiscreteField(
            pressureOp.resolutions[pressureOp.velocity])
        self.vorticity = pressureOp.vorticity.getDiscreteField(
            pressureOp.resolutions[pressureOp.vorticity])
        self.density = pressureOp.density.getDiscreteField(
            pressureOp.resolutions[pressureOp.density])
        self.compute_time = 0.

    @debug
    def setUp(self):
        ## La topologie se recupere depuis les variables discretes
        self.topology = self.velocity_old.topology
        self.topology = self.velocity_new.topology
        self.topology = self.vorticity.topology
        self.topology = self.vorticity.density

        self.resolution = self.topology.mesh.resolution
        self.meshSize = self.topology.mesh.size

    @debug
    def apply(self, t, dt, ite):
        """
        Apply operator.
        """
        self.compute_time = time.time()
        self.velocity_new = pressureOp.velocity.discreteField[IdVelocity]
        result = (self.velocity_new - self.velocity_old )/ dt
        gradU = DifferentialOperator_d(self.velocity_new, self.velocity_new, choice='gradV', topology=self.topology).apply()
        result = result + np.dot(self.velocity_new, gradU)
        ## result = - gradP/rho
        result = result - self.viscosity * DifferentialOperator_d(self.velocity_new, self.velocity_new, choice='laplacianV', topology=self.topology).apply()
        ## result = -(gradP/rho) x (gradRho/rho)
        result = np.cross(result, DifferentialOperator_d(self.density, self.density, choice='gradS', topology=self.topology).apply()/self.density)
        ## vorti(n+1) = vorti(n) + dt * result
        self.vorticity.data = self.vorticity.data + dt * result
        self.velocity_old = np.copy(self.velocity_new.data)
        self.compute_time = time.time() - self.compute_time
        return self.compute_time
#        return result

    def printComputeTime(self):
        self.timings_info[0] = "\"Pressure calculation total\""
#        self.timings_info[1] = str(self.total_time)
#        print "Multiphase total time : ", self.total_time
        print "Time of the last Dp/rho calculation loop :", self.compute_time

    def __str__(self):
        s = "Pressure_d (DiscreteOperator). " + DiscreteOperator.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Pressure_d"
    print Pressure_d.__doc__

