# -*- coding: utf-8 -*-
"""
@package parmepy.operator.Penalization

Penalization operator representation
"""
from parmepy.constants import debug
from parmepy.operator.continuous import Operator
from parmepy.mpi.topology import Cartesian
from parmepy.operator.penalization_d import Penalization_d


class Penalization(Operator):
    """
    Penalization operator representation
    """

    @debug
    def __init__(self, velocity, vorticity, obstacle, lambd,
                 resolutions=None, method='',
                 **other_config):
        """
        Constructor.
        Create a Penalization operator from given velocity variavle.

        @param velocity ContinuousVariable.ContinuousVariable : velocity variable.
        """
        Operator.__init__(self, [velocity, vorticity])
        ## obstacle to penalize
        self.obstacle = obstacle
        ## velocity variable to penalize
        self.velocity = velocity
        ## vorticity variable to penalize
        self.vorticity = vorticity
        ## lambda penalization function
        self.lambd = lambd
        self.resolutions = resolutions
        self.method = method
        self.config = other_config

    @debug
    def setUp(self):
        """
        Penalization operator discretization method.
        Create a PenalizationDOp.PenalizationDOp from given specifications.

        """
        Operator.setUp(self)
        print self.method
        #variables discretization
        for v in self.variables:
            topoId, topo = self.domain.addTopology(
                Cartesian(domain=self.domain, dim=self.domain.dimension,
                          globalMeshResolution=self.resolutions[v]))
            df, dfId = v.discretize(topo)
            self.discreteFieldId[v] = dfId
        self.discreteOperator = Penalization_d(self, method=self.method,
                                               **self.config)
        self.discreteOperator.setUp()

    def __str__(self):
        """ToString method"""
        s = " Penalization operator (ContinuousOperator)"
        if self.discreteOperator is not None:
            s += "with the following discretization:\n"
            s += str(self.discreteOperator)
        else:
            s += ". Not discretised"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Penalization"
    print Penalization.__doc__
