# -*- coding: utf-8 -*-
"""
@package parmepy.operator.scales_advection
Discrete Advection operator using scales code (Jean-Baptiste)
"""
from parmepy.f2py import scales2py
from parmepy.operator.discrete import DiscreteOperator
from parmepy.fields.vector import VectorField
from parmepy.constants import np, ORDER, PARMES_REAL, debug
import time


class ScalesAdvection(DiscreteOperator):
    """
    Operator representation.
    """

    @debug
    def __init__(self, advec, method=''):
        """
        Constructor.
        """
        self.advec = advec
        self.method = method
        self.velocity = self.advec.velocity.discreteField[
            self.advec.discreteFieldId[self.advec.velocity]]
        variables = [self.velocity]
        if not isinstance(self.advec.advectedFields, list):
            self.advectedFields = [self.advec.advectedFields.discreteField[
                self.advec.discreteFieldId[self.advec.advectedFields]]]
        else:
            self.advectedFields = [
                advecF.discreteField[self.advec.discreteFieldId[advecF]]
                for advecF in self.advec.advectedFields]
        [variables.append(advecF) for advecF in self.advectedFields]
        DiscreteOperator.__init__(self, variables,
                                  method)
        self.input = [self.velocity]
        self.output = list(variables).remove(self.velocity)
        self.compute_time = 0.

    @debug
    def setUp(self):
        pass
        #self.stab_coeff = stab_coeff
#        self.topology = self.velocity.topology
#        self.ghosts = self.topology.ghosts
#        self.resolution = self.topology.mesh.resolution
#        self.dim = self.topology.dim

    @debug
    def apply(self, t, dt, ite):
        """
        Apply operator.
        """
        self.compute_time = time.time()
        for adF in self.advectedFields:
            if isinstance(adF, VectorField):
                for d in xrange(adF.dimension):
                    adF[d][...] = scales2py.solve_advection(
                        dt, self.velocity.data[0],
                        self.velocity.data[1],
                        self.velocity.data[2],
                        adF[d][...])
            else:
                adF[...] = scales2py.solve_advection(
                    dt, self.velocity.data[0],
                    self.velocity.data[1],
                    self.velocity.data[2],
                    adF.data)

        # ind0a = self.ghosts[0]
        # ind0b = self.resolution[0] - self.ghosts[0]
        # ind1a = self.ghosts[1]
        # ind1b = self.resolution[1] - self.ghosts[1]
        # ind2a = self.ghosts[2]
        # ind2b = self.resolution[2] - self.ghosts[2]

        # velocityNoG = [np.zeros((self.resolution - 2 * self.ghosts),
        #                         dtype=PARMES_REAL, order=ORDER)
        #                for d in xrange(self.dim)]
        # for i in xrange(self.dim):
        #     velocityNoG[i][...] = self.velocity[i][ind0a:ind0b,
        #                                            ind1a:ind1b,
        #                                            ind2a:ind2b]
        # if self.advectedField.vector:

        #     advecFieldNoG = [np.zeros((self.resolution - 2 * self.ghosts),
        #                               dtype=PARMES_REAL, order=ORDER)
        #                      for d in xrange(self.dim)]
        #     for i in xrange(self.dim):
        #         advecFieldNoG[i][...] = self.advectedField[i][ind0a:ind0b,
        #                                                       ind1a:ind1b,
        #                                                      ind2a:ind2b]

        #     # Vector field advection
        #     advecFieldNoG[0][...] = \
        #         scales2py.solve_advection(dt, velocityNoG[0][...],
        #                                   velocityNoG[1][...],
        #                                   velocityNoG[2][...],
        #                                   advecFieldNoG[0][...])
        #     advecFieldNoG[1][...] = \
        #         scales2py.solve_advection(dt, velocityNoG[0][...],
        #                                   velocityNoG[1][...],
        #                                   velocityNoG[2][...],
        #                                   advecFieldNoG[1][...])
        #     advecFieldNoG[2][...] = \
        #         scales2py.solve_advection(dt, velocityNoG[0][...],
        #                                   velocityNoG[1][...],
        #                                   velocityNoG[2][...],
        #                                   advecFieldNoG[2][...])

        #     for i in xrange(self.dim):
        #         self.advectedField[i][ind0a:ind0b,
        #                               ind1a:ind1b, ind2a:ind2b] = \
        #             advecFieldNoG[i][...]

        #     # Computation of advected vector field norm --> addField :
        #     self.addField[ind0a:ind0b,
        #                   ind1a:ind1b, ind2a:ind2b] = \
        #         np.sqrt(advecFieldNoG[0][...] ** 2 + \
        #                 advecFieldNoG[1][...] ** 2 + \
        #                 advecFieldNoG[2][...] ** 2)

        #     addFieldNoG = np.zeros((self.resolution - 2 * self.ghosts),
        #                              dtype=PARMES_REAL, order=ORDER)
        #     addFieldNoG[...] = self.addField[ind0a:ind0b,
        #                                             ind1a:ind1b,
        #                                             ind2a:ind2b]

        #         # Added Scalar field advection
        #     addFieldNoG[...] = \
        #         scales2py.solve_advection(dt, velocityNoG[0][...],
        #                                   velocityNoG[1][...],
        #                                   velocityNoG[2][...],
        #                                   addFieldNoG[...])

        #     self.addField[ind0a:ind0b, ind1a:ind1b, ind2a:ind2b] = \
        #         addFieldNoG[...]


        # else:
        #     advecFieldNoG = np.zeros((self.resolution - 2 * self.ghosts),
        #                              dtype=PARMES_REAL, order=ORDER)
        #     advecFieldNoG[...] = self.advectedField[ind0a:ind0b,
        #                                             ind1a:ind1b,
        #                                             ind2a:ind2b]

        #     # Scalar field advection
        #     advecFieldNoG[...] = \
        #         scales2py.solve_advection(dt, velocityNoG[0][...],
        #                                   velocityNoG[1][...],
        #                                   velocityNoG[2][...],
        #                                   advecFieldNoG[...])

        #     self.advectedField[ind0a:ind0b, ind1a:ind1b, ind2a:ind2b] = \
        #         advecFieldNoG[...]

        self.compute_time = time.time() - self.compute_time
        self.total_time += self.compute_time
        return self.compute_time

    def printComputeTime(self):
        self.time_info[0] = "\"Advection calculation total\""
        self.time_info[1] = str(self.total_time)
        print "Time of the last advection calculation loop :", \
            self.compute_time

    def __str__(self):
        s = "Advec_scales_d (DiscreteOperator). " + \
            DiscreteOperator.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Advec_scales_d"
    print Advec_scales_d.__doc__
