# -*- coding: utf-8 -*-
import time
import parmepy as pp
import numpy as np
from parmepy.constants import PARMES_REAL, ORDER
from parmepy.domain.obstacle.obstacle_2D import Obstacle2D
from parmepy.fields.analytical import AnalyticalField
from parmepy.operator.penalization import Penalization
from parmepy.problem.navier_stokes import NSProblem
from parmepy.operator.monitors.printer import Printer


def computeVel(x, y, ):
    vx = 1.
    vy = 1.
    return vx, vy

def computeVort(x, y):
    wx = 0.
    wy = 0.
    return wx, wy

def test_Penal2D():
    # Parameters
    nb = 101
    dim = 2
    boxLength = [1., 1.]
    boxMin = [0., 0.]
    nbElem = [nb, nb]

    timeStep = 0.05
    finalTime = timeStep
    outputFilePrefix = './penal2D_'
    outputModulo = 1

    t0 = time.time()

    ## Domain
    box = pp.Box(dim, length=boxLength, origin=boxMin)

    ## Obstacle
    lambd = np.array([10.E8, 10.], 
                     dtype=PARMES_REAL, order=ORDER)
    cylinder = Obstacle2D(box, center=[0.5, 0.5], 
                          zlayer=0.02, 
                          porousLayerThickn=0.05, 
                          porousLayerConfig='', 
                          obstacleName='semi_cylinder', 
                          radius=0.2)

    ## Fields
    velo = AnalyticalField(domain=box, formula=computeVel, 
                           name='Velocity', vector=True)
    vorti = AnalyticalField(domain=box, formula=computeVort, 
                            name='Vorticity', vector=True)

    ## Operators
    penal = Penalization(velo, vorti,
                         obstacle=cylinder,
                         lambd=lambd,
                         resolutions={velo: nbElem,
                                      vorti: nbElem},
                         method='',
                         )

    ##Problem
    pb = NSProblem([penal],
                    monitors=[Printer(fields=[velo],
                              frequency=outputModulo,
                              outputPrefix=outputFilePrefix)])

    ## Setting solver to Problem
    pb.setUp(finalTime, timeStep)

    t1 = time.time()

    ## Solve problem
    timings = pb.solve()

    tf = time.time()


    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"
    print ""
#    return pb.timings_info[0] + "\"Solving time\" \"Init time\" \"Total time\"", pb.timings_info[1] + str(tf - t1) + " " + str(t1 - t0) + " " + str(tf - t0)

if __name__ == "__main__":
    test_Penal2D()
