# -*- coding: utf-8 -*-
import time

import parmepy as pp
import numpy as np
from parmepy.constants import PARMES_REAL, ORDER
from parmepy.fields.analytical import AnalyticalField
from parmepy.operator.poisson import Poisson
from parmepy.operator.diffusion import Diffusion
from parmepy.problem.navier_stokes import NSProblem
from parmepy.operator.monitors.energy_enstrophy import Energy_enstrophy
from math import sqrt, pi, exp


def computeVel(x, y):
    vx = 0.
    vy = 0.
    return vx, vy

def computeVort(x, y):
    w = np.cos(y) * np.cos(x)
    return w

def test_Diff_Poisson():
    # Parameters
    nb = 65
    dim = 2
    boxLength = [1., 1.]
    boxMin = [0., 0.]
    nbElem = [nb, nb]

    timeStep = 0.01
    finalTime = 150 * timeStep
    outputFilePrefix = './Energies'
    outputModulo = 10

    t0 = time.time()

    ## Domain
    box = pp.Box(dim, length=boxLength, origin=boxMin)

    ## Fields
    velo = AnalyticalField(domain=box, formula=computeVel, 
                           name='Velocity', vector=True)
    vorti = AnalyticalField(domain=box, formula=computeVort, 
                            name='Vorticity', vector=False)

    ## FFT Poisson solver
    poisson = Poisson(velo, vorti,
                      resolutions={velo: nbElem,
                                   vorti: nbElem},
                      method=''
                     )

    ## Problem
    pb = NSProblem([poisson])


    ## Setting solver to Problem
    pb.setUp(finalTime, timeStep)

    t1 = time.time()

    ## Solve problem
    timings = pb.solve()

    tf = time.time()

    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    test_Diff_Poisson()
