# -*- coding: utf-8 -*-
"""
@package tools
Communicator MPI to synchronize ghost
"""

import numpy as np
import mpi4py.MPI as MPI
import time


class Synchronize(object):
    """
    Synchronization of ghosts values on domain
    """

    def __init__(self, topology):
        """
        Constructor.
        @param topology : Local topology 
        """
        self.topo = topology
        self.total_time = 1
        ## problem le topology.mesh.resolution peut etre different du shape du field rentre
        self.size= topology.mesh.resolution


    def apply(self, *listFields):
        """
        Application of synchronization at a list of Fields
        @ list of Fields or one Fields.
        Make the synchronization with this law :
        for synchronize P1 with P2 :
            if P1.rank < P2.rank 
                P1 send, own and after recv ghost
                P2 recv ghost and send his own
            else 
                do the reciproque
        And we do that direction by direction, with a sort table of
        the synchro who need.
        """
#       Rappel de la correspondance direction <-> indice
#        self.topo.tabSort = np.array([[self.up,0] , [self.down,1] , [self.east,2], [self.west,3], 
#            [self.north,4],[self.south,5]])
        self.compute_time = time.time()
        self.comm = main_comm
        self.count = None

        for f in listFields:
#            self.size= f.data[0].shape
#            print 'size array:' ,self.size,  'size topo :',  self.topo.mesh.resolution
#            print 'rank', self.topo.rank, 'tab',self.topo.tabSort
            for i in xrange(self.topo.tabSort[:,0].shape[0]) :
#                print 'rang, destinataire, orientation', self.topo.rank, self.topo.tabSort[i,0], self.topo.tabSort[i,1]
                if ( self.topo.rank == self.topo.tabSort[i,0]):
                    # UP DOWN
                    if(self.topo.tabSort[i,1] == 0):
                        for j in xrange(self.topo.dim) :
                            f[j][self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:] =\
                            f[j][self.topo.ghosts[0]:2*self.topo.ghosts[0],:,:]

                    if(self.topo.tabSort[i,1] == 1):
                        for j in xrange(self.topo.dim) :
                            f[j][0:self.topo.ghosts[0],:,:] =\
                            f[j][self.topo.mesh.resolution[0]-2*self.topo.ghosts[0]:self.topo.mesh.resolution[0]-self.topo.ghosts[0],:,:]

                    # WEST EAST
                    if(self.topo.tabSort[i,1] == 2):
                        for j in xrange(self.topo.dim) :
                            f[j][:,self.topo.mesh.resolution[1]-self.topo.ghosts[1]:self.topo.mesh.resolution[1],:] =\
                            f[j][:,self.topo.ghosts[1]:2*self.topo.ghosts[1],:]

                    if(self.topo.tabSort[i,1] == 3):
                        for j in xrange(self.topo.dim) :
                            f[j][:,0:self.topo.ghosts[1],:] =\
                            f[j][:,self.topo.mesh.resolution[1]-2*self.topo.ghosts[1]:self.topo.mesh.resolution[1]-self.topo.ghosts[1],:]

                    # NORTH SOUTH
                    if(self.topo.tabSort[i,1] == 4):
                        for j in xrange(self.topo.dim) :
                            f[j][:,:,self.topo.mesh.resolution[2]-self.topo.ghosts[2]:self.topo.mesh.resolution[2]] =\
                            f[j][:,:,self.topo.ghosts[2]:2*self.topo.ghosts[2]]

                    if(self.topo.tabSort[i,1] == 5):
                        for j in xrange(self.topo.dim) :
                            f[j][:,:,0:self.topo.ghosts[2]] =\
                            f[j][:,:,self.topo.mesh.resolution[2]-2*self.topo.ghosts[2]:self.topo.mesh.resolution[2]-self.topo.ghosts[2]]
                else :
#                if(self.topo.tabSort[i,1] == 'up'):
                    if(self.topo.tabSort[i,1] == 0):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'UP',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.UPsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'UP', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.UPrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'down'):
                    if(self.topo.tabSort[i,1] == 1):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'DOWN',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.DOWNsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'DOWN', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.DOWNrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'east'):
                    if(self.topo.tabSort[i,1] == 2):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'EST',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.EASTsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'EST', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.EASTrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'west'):
                    if(self.topo.tabSort[i,1] == 3):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'WEST',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.WESTsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'WEST', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.WESTrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'north'):
                    if(self.topo.tabSort[i,1] == 4):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'NORTH',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.NORTHsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'NORTH', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.NORTHrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'south'):
                    if(self.topo.tabSort[i,1] == 5):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'SOUTH',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.SOUTHsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'SOUTH', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.SOUTHrecv(self.topo.tabSort[i,0], f)
        self.compute_time = time.time() - self.compute_time
        self.total_time += self.compute_time

        return self.compute_time


    def UPsend(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
                self.topo.mesh.resolution[0]-2*self.topo.ghosts[0]:self.topo.mesh.resolution[0]-self.topo.ghosts[0],:,:].shape)
            startsSend = np.asarray([self.topo.mesh.resolution[0]-2*self.topo.ghosts[0],0,0])
            UPtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            UPtypeSend.Create_contiguous( UPtypeSend.Get_true_extent()[1] + UPtypeSend.Get_true_extent()[0])
            UPtypeSend.Commit()
            self.comm.Send([listFields[i],self.count, UPtypeSend ], dest=proc, tag=11)
            UPtypeSend.Free()
            startsRecv = [self.topo.mesh.resolution[0]-self.topo.ghosts[0],0,0]
            UPtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            UPtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, UPtypeRecv] , source=proc, tag=11)
            listFields[i][
            self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:]\
            = data[self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:]
            UPtypeRecv.Free()


    def DOWNsend(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            self.topo.ghosts[0]:2*self.topo.ghosts[0],:,:].shape)
            startsSend = np.asarray([self.topo.ghosts[0],0,0])
            DOWNtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            DOWNtypeSend.Commit()
            self.comm.Send([ listFields[i], DOWNtypeSend ], dest=proc, tag=22)
            DOWNtypeSend.Free()
            startsRecv = [0,0,0]
            DOWNtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            DOWNtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, DOWNtypeRecv], source=proc, tag=22)
            listFields[i][0:self.topo.ghosts[0],:,:] = data[0:self.topo.ghosts[0],:,:]
            DOWNtypeRecv.Free()

    def EASTsend(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            :,self.topo.mesh.resolution[1]-2*self.topo.ghosts[1]:self.topo.mesh.resolution[1]-self.topo.ghosts[1],:].shape)
            startsSend = np.asarray([0,self.topo.mesh.resolution[1]-2*self.topo.ghosts[1],0])
            EASTtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI) 
            EASTtypeSend.Commit()
            self.comm.Send([ listFields[i], EASTtypeSend ], dest=proc, tag=33)
            EASTtypeSend.Free()
            startsRecv = [0,self.topo.mesh.resolution[1]-self.topo.ghosts[1],0]
            EASTtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            EASTtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, EASTtypeRecv] , source=proc, tag=33)
            listFields[i][
            :,self.topo.mesh.resolution[1]-self.topo.ghosts[1]:self.topo.mesh.resolution[1] ,:]\
            = data[:,self.topo.mesh.resolution[1]-self.topo.ghosts[1]:self.topo.mesh.resolution[1],:]
            EASTtypeRecv.Free()

    def WESTsend(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            :,self.topo.ghosts[1]:2*self.topo.ghosts[1],:].shape)
            startsSend = np.asarray([0,self.topo.ghosts[1],0])
            WESTtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            WESTtypeSend.Commit()
            self.comm.Send([ listFields[i], WESTtypeSend ], dest=proc, tag=44)
            WESTtypeSend.Free()
            startsRecv = [0,0,0]
            WESTtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            WESTtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, WESTtypeRecv], source=proc, tag=44)
            listFields[i][:,0:self.topo.ghosts[1],:] = data[:,0:self.topo.ghosts[1],:]
            WESTtypeRecv.Free()


    def NORTHsend(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
                :,:,self.topo.mesh.resolution[2]-2*self.topo.ghosts[2]:self.topo.mesh.resolution[2]-self.topo.ghosts[2]].shape)
            startsSend = np.asarray([0,0,self.topo.mesh.resolution[2]-2*self.topo.ghosts[2]])
            NORTHtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            NORTHtypeSend.Commit()
            self.comm.Send([listFields[i], NORTHtypeSend ], dest=proc, tag=55)
            NORTHtypeSend.Free()
            startsRecv = [0,0,self.topo.mesh.resolution[2]-self.topo.ghosts[2]]
            NORTHtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            NORTHtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, NORTHtypeRecv] , source=proc, tag=55)
            listFields[i][
            :,:,self.topo.mesh.resolution[2]-self.topo.ghosts[2]:self.topo.mesh.resolution[2]]\
            = data[:,:,self.topo.mesh.resolution[2]-self.topo.ghosts[2]:self.topo.mesh.resolution[2]]
            NORTHtypeRecv.Free()

    def SOUTHsend(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            :,:,self.topo.ghosts[2]:2*self.topo.ghosts[2]].shape)
            startsSend = np.asarray([0,0,self.topo.ghosts[2]])
            SOUTHtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            SOUTHtypeSend.Commit()
            self.comm.Send([ listFields[i], SOUTHtypeSend ], dest=proc, tag=66)
            SOUTHtypeSend.Free()
            startsRecv = [0,0,0]
            SOUTHtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            SOUTHtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, SOUTHtypeRecv], source=proc, tag=66)
            listFields[i][:,:,0:self.topo.ghosts[2]] = data[:,:,0:self.topo.ghosts[2]]
            SOUTHtypeRecv.Free()

    def UPrecv(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:].shape)
            startsRecv = np.asarray([self.topo.mesh.resolution[0]-self.topo.ghosts[0],0,0])
            UPtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI) 
            UPtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, UPtypeRecv], source=proc, tag=22)
            UPtypeRecv.Free()
            listFields[i][self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:] = data[ \
            self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:]
            startsSend = [self.topo.mesh.resolution[0]-2*self.topo.ghosts[0],0,0]
            UPtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            UPtypeSend.Commit()
            self.comm.Send([listFields[i], UPtypeSend], dest=proc, tag=22)
            UPtypeSend.Free()


    def DOWNrecv(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            0:self.topo.ghosts[0],:,:].shape)
            startsRecv = np.asarray([0,0,0])
            data=listFields[i]
            DOWNtypeRecv= MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            DOWNtypeRecv.Create_contiguous( DOWNtypeRecv.Get_true_extent()[1] + DOWNtypeRecv.Get_true_extent()[0])
            DOWNtypeRecv.Commit()
            self.comm.Recv([data, self.count , DOWNtypeRecv], source = proc, tag =11)
            DOWNtypeRecv.Free()
            listFields[i][0:self.topo.ghosts[0],:,:] = data[0:self.topo.ghosts[0],:,:]
            startsSend = [self.topo.ghosts[0],0,0]
            DOWNtypeSend= MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            DOWNtypeSend.Commit()
            self.comm.Send([listFields[i], DOWNtypeSend], dest = proc, tag =11) 
            DOWNtypeSend.Free()


    def EASTrecv(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            :,self.topo.mesh.resolution[1]-self.topo.ghosts[1]:self.topo.mesh.resolution[1],:].shape)
            startsRecv = np.asarray([0,self.topo.mesh.resolution[1]-self.topo.ghosts[1],0])
            EASTtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI) 
            EASTtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, EASTtypeRecv], source=proc, tag=44)
            EASTtypeRecv.Free()
            listFields[i][:,self.topo.mesh.resolution[1]-self.topo.ghosts[1]:self.topo.mesh.resolution[1],:] = data[ \
            :,self.topo.mesh.resolution[1]-self.topo.ghosts[1]:self.topo.mesh.resolution[1],:]
            startsSend = [0,self.topo.mesh.resolution[1]-2*self.topo.ghosts[1],0]
            EASTtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            EASTtypeSend.Commit()
            self.comm.Send([listFields[i], EASTtypeSend], dest=proc, tag=44)
            EASTtypeSend.Free()

    def WESTrecv(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            :,0:self.topo.ghosts[1],:].shape)
            startsRecv = np.asarray([0,0,0])
            WESTtypeRecv= MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            WESTtypeRecv.Commit()
            data=listFields[i]
            self.comm.Recv([data, WESTtypeRecv], source = proc, tag =33)
            WESTtypeRecv.Free()
            listFields[i][:,0:self.topo.ghosts[1],:] = data[:,0:self.topo.ghosts[1],:]
            startsSend = [0,self.topo.ghosts[1],0]
            WESTtypeSend= MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            WESTtypeSend.Commit()
            self.comm.Send([listFields[i], WESTtypeSend], dest = proc, tag =33) 
            WESTtypeSend.Free()

    def NORTHrecv(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            :,:,self.topo.mesh.resolution[2]-self.topo.ghosts[2]:self.topo.mesh.resolution[2]].shape)
            startsRecv = np.asarray([0,0,self.topo.mesh.resolution[2]-self.topo.ghosts[2]])
            NORTHtypeRecv = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI) 
            NORTHtypeRecv.Commit()
            data = listFields[i]
            self.comm.Recv([data, NORTHtypeRecv], source=proc, tag=66)
            NORTHtypeRecv.Free()
            listFields[i][:,:,self.topo.mesh.resolution[2]-self.topo.ghosts[2]:self.topo.mesh.resolution[2]] = data[ \
            :,:,self.topo.mesh.resolution[2]-self.topo.ghosts[2]:self.topo.mesh.resolution[2]]
            startsSend = [0,0,self.topo.mesh.resolution[2]-2*self.topo.ghosts[2]]
            NORTHtypeSend = MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            NORTHtypeSend.Commit()
            self.comm.Send([listFields[i], NORTHtypeSend], dest=proc, tag=66)
            NORTHtypeSend.Free()

    def SOUTHrecv(self, proc, listFields):
        for i in xrange(self.topo.dim) :
            subsizes = np.asarray(listFields[i][ \
            :,:,0:self.topo.ghosts[2]].shape)
            startsRecv = np.asarray([0,0,0])
            SOUTHtypeRecv= MPI.DOUBLE.Create_subarray(self.size, subsizes, startsRecv, order=ORDERMPI)
            SOUTHtypeRecv.Commit()
            data=listFields[i]
            self.comm.Recv([data, SOUTHtypeRecv], source = proc, tag =55)
            SOUTHtypeRecv.Free()
            listFields[i][:,:,0:self.topo.ghosts[2]] = data[:,:,0:self.topo.ghosts[2]]
            startsSend = [0,0,self.topo.ghosts[2]]
            SOUTHtypeSend= MPI.DOUBLE.Create_subarray(self.size, subsizes, startsSend, order=ORDERMPI)
            SOUTHtypeSend.Commit()
            self.comm.Send([listFields[i], SOUTHtypeSend], dest = proc, tag =55) 
            SOUTHtypeSend.Free()


    def printComputeTime(self):
        self.timings_info[0] = "\"Synchronization total\""
        self.timings_info[1] = str(self.total_time)
        print "Synchronization total time : ", self.total_time
        print "Time of the last Synchronization iteration :", self.compute_time

    def __str__(self):
        s = "Synchronize. "
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Synchronize"
    print Synchronize.__doc__
