# -*- coding: utf-8 -*-
"""
@package tools
Communicator MPI to synchronize ghost
"""
from parmepy.constants import ORDER
import numpy as np
from parmepy.mpi import main_comm
import time


class Synchronize(object):
    """
    Synchronization of ghosts values on domain
    """

    def __init__(self, topology):
        """
        Constructor.
        @param topology : Local topology 
        """
        self.topo = topology
        self.total_time = 1
        ## problem le topology.mesh.resolution peut etre different du shape du field rentre
        self.size= topology.mesh.resolution


    def apply(self, *listFields):
        """
        Application of synchronization at a list of Fields
        @ list of Fields or one Fields.
        Make the synchronization with this law :
        for synchronize P1 with P2 :
            if P1.rank < P2.rank 
                P1 send, own and after recv ghost
                P2 recv ghost and send his own
            else 
                do the reciproque
        And we do that direction by direction, with a sort table of
        the synchro who need.
        """
#       Rappel de la correspondance direction <-> indice
#        self.topo.tabSort = np.array([[self.up,0] , [self.down,1] , [self.east,2], [self.west,3], 
#            [self.north,4],[self.south,5]])
        self.compute_time = time.time()
        self.comm = main_comm
        self.count = None

        for f in listFields:
#            self.size= f.data[0].shape
#            print 'size array:' ,self.size,  'size topo :',  self.topo.mesh.resolution
#            print 'rank', self.topo.rank, 'tab',self.topo.tabSort
            for i in xrange(self.topo.tabSort[:,0].shape[0]) :
#                print 'rang, destinataire, orientation', self.topo.rank, self.topo.tabSort[i,0], self.topo.tabSort[i,1]
                if ( self.topo.rank == self.topo.tabSort[i,0]):
                    # UP DOWN
                    if(self.topo.tabSort[i,1] == 0):
                        for j in xrange(self.topo.dim) :
                            f[j][self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:] =\
                            f[j][self.topo.ghosts[0]:2*self.topo.ghosts[0],:,:]

                    if(self.topo.tabSort[i,1] == 1):
                        for j in xrange(self.topo.dim) :
                            f[j][0:self.topo.ghosts[0],:,:] =\
                            f[j][self.topo.mesh.resolution[0]-2*self.topo.ghosts[0]:self.topo.mesh.resolution[0]-self.topo.ghosts[0],:,:]

                    # WEST EAST
                    if(self.topo.tabSort[i,1] == 2):
                        for j in xrange(self.topo.dim) :
                            f[j][:,self.topo.mesh.resolution[1]-self.topo.ghosts[1]:self.topo.mesh.resolution[1],:] =\
                            f[j][:,self.topo.ghosts[1]:2*self.topo.ghosts[1],:]

                    if(self.topo.tabSort[i,1] == 3):
                        for j in xrange(self.topo.dim) :
                            f[j][:,0:self.topo.ghosts[1],:] =\
                            f[j][:,self.topo.mesh.resolution[1]-2*self.topo.ghosts[1]:self.topo.mesh.resolution[1]-self.topo.ghosts[1],:]

                    # NORTH SOUTH
                    if(self.topo.tabSort[i,1] == 4):
                        for j in xrange(self.topo.dim) :
                            f[j][:,:,self.topo.mesh.resolution[2]-self.topo.ghosts[2]:self.topo.mesh.resolution[2]] =\
                            f[j][:,:,self.topo.ghosts[2]:2*self.topo.ghosts[2]]

                    if(self.topo.tabSort[i,1] == 5):
                        for j in xrange(self.topo.dim) :
                            f[j][:,:,0:self.topo.ghosts[2]] =\
                            f[j][:,:,self.topo.mesh.resolution[2]-2*self.topo.ghosts[2]:self.topo.mesh.resolution[2]-self.topo.ghosts[2]]
                else :
#                if(self.topo.tabSort[i,1] == 'up'):
                    if(self.topo.tabSort[i,1] == 0):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'UP',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.UPsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'UP', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.UPrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'down'):
                    if(self.topo.tabSort[i,1] == 1):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'DOWN',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.DOWNsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'DOWN', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.DOWNrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'east'):
                    if(self.topo.tabSort[i,1] == 2):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'EST',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.EASTsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'EST', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.EASTrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'west'):
                    if(self.topo.tabSort[i,1] == 3):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'WEST',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.WESTsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'WEST', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.WESTrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'north'):
                    if(self.topo.tabSort[i,1] == 4):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'NORTH',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.NORTHsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'NORTH', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.NORTHrecv(self.topo.tabSort[i,0], f)
#                    if(self.topo.tabSort[i,1] == 'south'):
                    if(self.topo.tabSort[i,1] == 5):
                        if( self.topo.rank < self.topo.tabSort[i,0]) :
#                            print 'SOUTH',self.topo.rank,'Send to',self.topo.tabSort[i,0]
                            self.SOUTHsend(self.topo.tabSort[i,0], f)
                        else :
#                            print 'SOUTH', self.topo.rank,'Recv from',self.topo.tabSort[i,0]
                            self.SOUTHrecv(self.topo.tabSort[i,0], f)
        self.compute_time = time.time() - self.compute_time
        self.total_time += self.compute_time

        return self.compute_time


    def UPsend(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((ghosts[0],resolution[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            data = np.array(np.copy(listFields[i][ \
                self.topo.mesh.resolution[0]-2*self.topo.ghosts[0]:self.topo.mesh.resolution[0]-self.topo.ghosts[0],:,:]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=11)
            self.comm.Recv(data , source=proc, tag=11)
            listFields[i][
            self.topo.mesh.resolution[0]-self.topo.ghosts[0]:self.topo.mesh.resolution[0],:,:]\
            = data


    def DOWNsend(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((ghosts[0],resolution[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            data = np.array(np.copy(listFields[i][ \
                ghosts[0]:2*ghosts[0],:,:]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=22)
            self.comm.Recv(data , source=proc, tag=22)
            listFields[i][0:ghosts[0],:,:] = data


    def EASTsend(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], ghosts[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            data = np.array(np.copy(listFields[i][ \
                :,resolution[1]-2*ghosts[1]:resolution[1]-ghosts[1],:]), order=ORDER)

            self.comm.Ssend(data, dest=proc, tag=33)
            self.comm.Recv(data , source=proc, tag=33)
            listFields[i][:,resolution[1]-ghosts[1]:resolution[1],:]= data


    def WESTsend(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], ghosts[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            data = np.array(np.copy(listFields[i][ \
                :,ghosts[1]:ghosts[1]*2,:]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=44)
            self.comm.Recv(data , source=proc, tag=44)
            listFields[i][:,0:ghosts[1],:]= data


    def NORTHsend(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], resolution[1], ghosts[2]) , dtype=PARMES_REAL , order=ORDER)
            data = np.array(np.copy(listFields[i][ \
                :,:,resolution[2]-2*ghosts[2]:resolution[2]-ghosts[2]]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=55)
            self.comm.Recv(data , source=proc, tag=55)
            listFields[i][:,:,resolution[2]-ghosts[2]:resolution[2]]= data


    def SOUTHsend(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], resolution[1], ghosts[2]) , dtype=PARMES_REAL , order=ORDER)
            data = np.array(np.copy(listFields[i][ \
                :,:,ghosts[2]:2*ghosts[2]]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=66)
            self.comm.Recv(data , source=proc, tag=66)
            listFields[i][:,:,0:ghosts[2]]= data


    def UPrecv(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((ghosts[0],resolution[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            self.comm.Recv(data, source = proc, tag =22)
            listFields[i][resolution[0]-ghosts[0]:resolution[0],:,:] = data
            data = np.array(np.copy(listFields[i][ \
                resolution[0]-2*ghosts[0]:resolution[0]-ghosts[0],:,:]), order=ORDER)
            self.comm.Ssend(data, dest = proc, tag =22)


    def DOWNrecv(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((ghosts[0],resolution[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            self.comm.Recv(data, source = proc, tag =11)
            listFields[i][0:self.topo.ghosts[0],:,:] = data
            data = np.array(np.copy(listFields[i][ \
                ghosts[0]:ghosts[0]*2,:,:]), order=ORDER)
            self.comm.Ssend(data, dest = proc, tag =11)  


    def EASTrecv(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], ghosts[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            self.comm.Recv(data , source=proc, tag=44)
            listFields[i][:,resolution[1]-ghosts[1]:resolution[1],:]= data
            data = np.array(np.copy(listFields[i][ \
                :,resolution[1]-2*ghosts[1]:resolution[1]-ghosts[1],:]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=44)


    def WESTrecv(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], ghosts[1], resolution[2]) , dtype=PARMES_REAL , order=ORDER)
            self.comm.Recv(data , source=proc, tag=33)
            listFields[i][:,0:ghosts[1],:]= data
            data = np.array(np.copy(listFields[i][ \
                :,ghosts[1]:2*ghosts[1],:]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=33)


    def NORTHrecv(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], resolution[1], ghosts[2]) , dtype=PARMES_REAL , order=ORDER)
            self.comm.Recv(data , source=proc, tag=66)
            listFields[i][:,:,resolution[2]-ghosts[2]:resolution[2]]= data
            data = np.array(np.copy(listFields[i][ \
                :,:,resolution[2]-2*ghosts[2]:resolution[2]-ghosts[2]]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=66)


    def SOUTHrecv(self, proc, listFields):
        ghosts= self.topo.ghosts
        resolution = self.topo.mesh.resolution
        for i in xrange(self.topo.dim) :
            data = np.zeros((resolution[0], resolution[1], ghosts[2]) , dtype=PARMES_REAL , order=ORDER)
            self.comm.Recv(data , source=proc, tag=55)
            listFields[i][:,:,0:ghosts[2]]= data
            data = np.array(np.copy(listFields[i][ \
                :,:,ghosts[2]:2*ghosts[2]]), order=ORDER)
            self.comm.Ssend(data, dest=proc, tag=55)


    def printComputeTime(self):
        self.timings_info[0] = "\"Synchronization total\""
        self.timings_info[1] = str(self.total_time)
        print "Synchronization total time : ", self.total_time
        print "Time of the last Synchronization iteration :", self.compute_time

    def __str__(self):
        s = "Synchronize. "
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Synchronize"
    print Synchronize.__doc__
